
/******************************************************************************

 The most basic example program to write a line of text in to the terminal
 using the USART library for pic16 mcus.

 Compiler: Microchip XC8 v1.12 (http://www.microchip.com/xc)
 IDE: Microchip MPLABX

 MCU: PIC16F877A
 Frequency: 20MHz

                                     NOTICE

NO PART OF THIS WORK CAN BE COPIED, DISTRIBUTED OR PUBLISHED WITHOUT A
WRITTEN PERMISSION FROM EXTREME ELECTRONICS INDIA. THE LIBRARY, NOR ANY PART
OF IT CAN BE USED IN COMMERCIAL APPLICATIONS. IT IS INTENDED TO BE USED FOR
HOBBY, LEARNING AND EDUCATIONAL PURPOSE ONLY. IF YOU WANT TO USE THEM IN
COMMERCIAL APPLICATION PLEASE WRITE TO THE AUTHOR.


WRITTEN BY:
AVINASH GUPTA
me@avinashgupta.com

 *******************************************************************************/

#include <xc.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "usart_pic16.h"
__EEPROM_DATA(30,0,5,0,0,0,0,0);
// Configuration Byte

#define _XTAL_FREQ  4000000      // oscillator frequency for _delay()

//#pragma config FOSC = HS        // With this, WDT doesn't work. Oscillator Selection bits (HS oscillator)
#pragma config WDTE = OFF       // Watchdog Timer Enable bit (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable bit (PWRT disabled)
#pragma config BOREN = ON       // Brown-out Reset Enable bit (BOR enabled)
//#pragma config LVP = ON         // Low-Voltage (Single-Supply) In-Circuit Serial Programming Enable bit (RB3/PGM pin has PGM function; low-voltage programming enabled)
#pragma config CPD = OFF        // Data EEPROM Memory Code Protection bit (Data EEPROM code protection off)
//#pragma config WRT = OFF        // Flash Program Memory Write Enable bits (Write protection off; all program memory may be written to by EECON control)
#pragma config CP = OFF         // Flash Program Memory Code Protection bit (Code protection off)

#define AUTO_ONL 0

#define AUTO_OFFL 2

#define AUTO_EN 4


int local_out=0;
 char tmp[48];
 unsigned int on_after_sec = 0;

 typedef struct {
    char *cmdstr;
    void (*func)(char *cmd);
   char *ex;
} cmd_t;
void ra0_on(char *);
void ra1_on(char *);
void ra0_off(char *);
void ra1_off(char *);
void set_auto(char *);
void on_auto(char *);
void off_auto(char *);
void dump_eep(char *);
void help(char *);
void soft_reset(char *);
void set_on_after_sec(char *);


const cmd_t functbl[] = {
    {"ra0_on", ra0_on, ""},
    {"ra0_off", ra0_off, ""},
    {"ra1_on", ra1_on, ""},
    {"ra1_off", ra1_off, ""},
    {"reset", soft_reset, ""},
    {"set_auto", set_auto, " 0030 0005"},
    {"on_auto", on_auto, ""},
    {"off_auto", off_auto, ""},
    {"dump_eep", dump_eep, ""},
    {"set__on", set_on_after_sec, " 0060 (sec)" }, 
    

    {"help", help, ""},
};
#define FUNCTBL_SIZE (sizeof(functbl)/ sizeof(functbl[0]))
 void delay_ms(unsigned long milliseconds) {
    while (milliseconds > 0) {
        if (milliseconds >= 10) {
            __delay_ms(10);
            milliseconds -= 10;
        } else {
            __delay_ms(1);
            milliseconds--;
        }
    }
}
 
char *my_strstr(char * s1, char * s2) {
    while (s1 && *s1) {
        if (strncmp(s1, s2, strlen(s2)) == 0)
            return (char *) s1;
        s1 = strchr(s1 + 1, *s2);
    }
    return NULL;
}
void soft_reset(char *cmd) {

    WDTCON=1;
    while(1);
    
}
void set_on_after_sec(char *cmd)
{
    int v1;
    memcpy(tmp, cmd + 9, 4);
    tmp[4] = 0;
    v1 = atoi(tmp);
    on_after_sec = v1;
    
}
void ra0_op(int onoff) {
    if (onoff)
        local_out |= 1;
    else
        local_out &= (~1);
    PORTA = local_out;
}

void ra1_op(int onoff) {


    if (onoff)
        local_out |= 2;
    else
        local_out &= (~2);
    PORTA = local_out;

}
void ra0_on(char *cmd) 
{
    ra0_op(1);
}
void ra0_off(char *cmd) 
{
    ra0_op(0);
}
void ra1_on(char *cmd) 
{
    ra1_op(1);
}
void ra1_off(char *cmd) 
{
    ra1_op(0);
}
void set_auto(char *cmd) {
    int v1, v2;
    memcpy(tmp, cmd + 9, 4);
    tmp[4] = 0;
    v1 = atoi(tmp);
    memcpy(tmp, cmd + 14, 4);
    tmp[4] = 0;
    v2 = atoi(tmp);
    USARTGotoNewLine();
    USARTWriteString("Auto on ");
    sprintf(tmp, "%d seconds", v1);
    USARTWriteString(tmp);
    USARTWriteString("off ");
    sprintf(tmp, "%d seconds", v2);
    USARTWriteString(tmp);
    eeprom_write(AUTO_ONL, v1 & 0xFF);
    eeprom_write(AUTO_ONL + 1, (v1 >> 8) & 0xFF);
    eeprom_write(AUTO_OFFL, v2 & 0xFF);
    eeprom_write(AUTO_OFFL + 1, (v2 >> 8) & 0xFF);
}
void on_auto(char *cmd)
{
    eeprom_write(AUTO_EN,1);
}
void off_auto(char *cmd)
{
    eeprom_write(AUTO_EN,0);
}

void dump_eep(char *cmd) {
    int i;
    for (i = 0; i < 8; i++) {
        sprintf(tmp, "i%d=0x%02x ", i, eeprom_read(i));
        USARTWriteLine(tmp);
    }
}
void help(char *cmd) 
{
    int i;
    for(i=0;i<FUNCTBL_SIZE;i++) {
        USARTWriteString(functbl[i].cmdstr);
        USARTWriteString("   ");
        USARTWriteString(functbl[i].cmdstr);
        USARTWriteString(functbl[i].ex);
        USARTGotoNewLine();
    }
}

void check_func(char *cmd) 
{
    int i;
    for(i=0;i<FUNCTBL_SIZE;i++) {
        if(my_strstr(cmd, functbl[i].cmdstr)) {
            sprintf(tmp, "Run %s", functbl[i].cmdstr);
            USARTWriteLine(tmp);
            USARTGotoNewLine();
            functbl[i].func(cmd);
            break;
        }
    }
    if(i >=FUNCTBL_SIZE) {
        USARTWriteLine("'help' to show usage.");
        USARTGotoNewLine();
    }
#if 0
    int v1,v2,i;
    if (strstr((const char *)cmd, "ra0_on")) {
        USARTWriteLine("RA0_PowerOn");
        ra0_op(1);
    } else if (my_strstr((const char *) cmd, "ra0_off")) {
        USARTWriteLine("RA0_PowerOff");
        ra0_op(0);
    } else if (my_strstr((const char *) cmd, "ra1_on")) {
        USARTWriteLine("RA1_PowerOn");
        ra1_op(1);
    } else if (my_strstr((const char *) cmd, "ra1_off")) {
        USARTWriteLine("RA1_PowerOff");
        ra1_op(0);
    } else if (my_strstr((const char *) cmd, "set_auto")) {
  
        memcpy(tmp, cmd+9, 3);
        tmp[3]=0;
        v1=atoi(tmp);
        memcpy(tmp, cmd+13, 3);
        tmp[3]=0;
        v2=atoi(tmp);
        USARTGotoNewLine();
        USARTWriteString("Auto on ");
        sprintf(tmp, "%d seconds", v1);
        USARTWriteString(tmp);
        USARTWriteString("off ");
        sprintf(tmp, "%d seconds", v2);
        USARTWriteString(tmp);
        eeprom_write(AUTO_ONL, v1 & 0xFF);
        eeprom_write(AUTO_ONL+1, (v1 >> 8) & 0xFF);
        eeprom_write(AUTO_OFFL, v2 & 0xFF);
        eeprom_write(AUTO_OFFL+1, (v2 >> 8) & 0xFF);
    } else if (my_strstr((const char *) cmd, "on_auto")) {
        eeprom_write(AUTO_EN,1);
    } else if (my_strstr((const char *) cmd, "off_auto")) {
        eeprom_write(AUTO_EN, 0);
    } else if (my_strstr((const char *) cmd, "dump_eep")) {
        for (i = 0; i < 8; i++) {
            sprintf(tmp, "i%d=0x%02x ", i, eeprom_read(i));
            USARTWriteLine(tmp);
        }
    } else {
        USARTWriteLine("Command not found.");
    }

#endif
}

void ra01_init() {
    int v;
    v = TRISA;
    v &= ~(3);
    TRISA = v;
}


 
 




void main() {
    char cmd[19];
 int cnt=0;
    int i = 0;
    int no_input_cnt = 0;
    char any_input = 0;
    char auto_en = eeprom_read(AUTO_EN);
    int auto_on = eeprom_read(AUTO_ONL) + (eeprom_read(AUTO_ONL+1) << 8);
    int auto_off = eeprom_read(AUTO_OFFL) + (eeprom_read(AUTO_OFFL+1) << 8);
    int on_after_sec_fraction=0;
    
    
    

    //Initialize USART with baud rate 9600
    ANS11 = 0;
    checkError();
    USARTInit(9600);

    char data;


    //Write intro text
    /*
    USARTWriteLine("USART Test");
    USARTWriteLine("----------");
    USARTWriteLine("Type a character on keyboard");
    USARTWriteLine("it will reach the PIC MCU via the serial line");
    USARTWriteLine("PIC MCU will return the same character but ");
    USARTWriteLine("enclosed in a <>");
    USARTWriteLine("--");
    USARTWriteLine("For example if you type 'a' you will see <a>");
    USARTWriteLine("appear on serial terminal.");
    USARTWriteLine(" ");
    USARTWriteLine("This checks both way serial transfers.");
    USARTWriteLine("");
    USARTWriteLine("Copyright (C) 2008-2013");
    USARTWriteLine("www.eXtremeElectronics.co.in");
     * */
    //USARTGotoNewLine();
    //USARTGotoNewLine();

    // USARTWriteLine("***********************************************");
    cmd[0] = 0;
#if 0 //PORTC ???OK
    PORTC = 0x00;
    TRISC = 0x00;
    CM2CON0 = 0;
    while (1) {
        PORTC = 0;
        delay_ms(1000);
        PORTC = 0xff;
        delay_ms(1000);
    }
#endif
    
    ra01_init();
    ra0_op(1); ra1_op(1);
    delay_ms(500);
    ra0_op(0);ra1_op(0);            
    delay_ms(500);    
    ra0_op(1); ra1_op(1);
    delay_ms(500);
    ra0_op(0);ra1_op(0);            

        USARTWriteLine("== USB On/Off Controller =="); 


    while (1) {
        //Get the amount of data waiting in USART queue
        uint8_t n = USARTDataAvailable();
#if 1
        if(on_after_sec) {
            delay_ms(1);
            if(on_after_sec_fraction) 
                on_after_sec_fraction--;
            if(on_after_sec_fraction == 0) {
                on_after_sec--;
                if(on_after_sec)
                    on_after_sec_fraction=1000;
            }
            if(on_after_sec == 0) {
                ra1_op(1);
                sprintf(tmp, "timed up! turn on ra1");
                USARTWriteLine(tmp);
                
            }
        }
#endif
        if((any_input == 0) && (auto_on > 0) && (auto_off > 0) && (auto_en == 1)) {
            if((no_input_cnt % 100) == 0) {
                sprintf(tmp, "idlecnt %d,on %ds off %ds", no_input_cnt, auto_on, auto_off);
                USARTWriteLine(tmp);
            }
            while((auto_en == 1) && (no_input_cnt > 15000) ) {
               
                cnt++;
                sprintf(tmp, "Now on (%d). on %ds off %ds", cnt, auto_on, auto_off);
                USARTWriteLine(tmp);
                ra1_op(1);
                delay_ms(auto_on * 1000);
                sprintf(tmp, "Now off (%d). on %ds off %ds", cnt, auto_on, auto_off);
                USARTWriteLine(tmp);
                ra1_op(0);
                delay_ms(auto_off * 1000);
            }
        }
        //If we have some data

        if (n > 0) {

            any_input = 1;
            checkError();
            data = USARTReadData();
            if ((data == '\r') || (data == '\n')) {
                USARTGotoNewLine();
                cmd[i] = 0;
                check_func(cmd);
                i = 0;
                cmd[0] = 0;
            } else {
                USARTWriteChar(data);
                cmd[i] = data;
                i++;
                cmd[i] = 0;
                if (i == (sizeof(cmd)-1)) {
                    check_func(cmd);
                    i = 0;
                    cmd[0] = 0;

                }
            }

        }
        else
            no_input_cnt++;
/*
        if (RCIF && 0) {

            //Read it
            PIR1bits.RCIF = 0;
            data = RCREG;
            checkError();
            //And send back
            if ((data == '\r') || (data == '\n'))
                USARTGotoNewLine();
            else
                USARTWriteChar(data);
        }
*/

    }
}
